/*
 * Copyright (c) Institute for System Programming of the Russian Academy of Sciences
 * All Rights Reserved
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISP RAS)
 * 25 Alexander Solzhenitsyn st., Moscow, 109004, Russia
 * http://www.ispras.ru
 *
 * The present file was automatically generated on the basis of formal specifications.
 * It is distributed under the same terms and conditions as the derived specifications.
 *
 * N.B. PLEASE DO NOT MODIFY THIS FILE.
 */

package ru.ispras.microtesk.model.plasma.op;

import java.util.Map;
import java.math.BigInteger;
import ru.ispras.microtesk.model.api.ArgumentMode;
import ru.ispras.microtesk.model.api.Execution;
import ru.ispras.microtesk.model.api.data.*;
import ru.ispras.microtesk.model.api.memory.*;
import ru.ispras.microtesk.model.api.instruction.*;
import ru.ispras.microtesk.model.api.memory.Location;
import ru.ispras.microtesk.model.plasma.mode.*;

import static ru.ispras.microtesk.model.plasma.Shared.*;

public final class mips32_load extends Operation {
  private static final class Info extends InfoAndRule {
    Info() {
      super(
         mips32_load.class,
         "mips32_load",
         new ArgumentDecls()
             .add("load_op", Type.CARD(3))
             .add("signed_op", BYTE)
             .add("rt", R.INFO)
             .add("load_offset", SHORT)
             .add("load_base", R.INFO));
    }

    @Override
    public Operation create(final Map<String, Primitive> args) {
      final Immediate load_op = (Immediate) getArgument("load_op", args);
      final Immediate signed_op = (Immediate) getArgument("signed_op", args);
      final R rt = (R) getArgument("rt", args);
      final Immediate load_offset = (Immediate) getArgument("load_offset", args);
      final R load_base = (R) getArgument("load_base", args);

      return new mips32_load(load_op, signed_op, rt, load_offset, load_base);
    }
  }
  public static final IInfo INFO = new Info();

  public final Immediate load_op;
  public final Immediate signed_op;
  public final R rt;
  public final Immediate load_offset;
  public final R load_base;

  public mips32_load(final Immediate load_op, final Immediate signed_op, final R rt, final Immediate load_offset, final R load_base) {
    assert Type.CARD(3).equals(load_op.getType());
    assert BYTE.equals(signed_op.getType());
    assert R.INFO.isSupported(rt);
    assert SHORT.equals(load_offset.getType());
    assert R.INFO.isSupported(load_base);
    this.load_op = load_op;
    this.signed_op = signed_op;
    this.rt = rt;
    this.load_offset = load_offset;
    this.load_base = load_base;
  }

  @Override
  public void action() {
    temp_address_load.access().store(load_base.access().load().add(Data.signExtend(WORD, load_offset.access().load())));
    Execution.trace("temp_address_load = 0x%x", temp_address_load.access().load().bigIntegerValue(false));
    Execution.trace("temp_address_load = 0x%x", temp_address_load.access().load().shiftRight(Data.valueOf(WORD, 0x2)).bigIntegerValue(false));
    load_temp32.access().store(MEM.access(temp_address_load.access().load().shiftRight(Data.valueOf(WORD, 0x2))).load());
    Execution.trace("MEM = 0x%x", load_temp32.access().load().bigIntegerValue(false));
    temp_byte_offset.access().store(temp_address_load.access().bitField(1, 0).load().xor(Data.valueOf(Type.CARD(2), 0x0)));
    load_address_error.access().store(Data.zeroExtend(WORD, temp_byte_offset.access().bitField(1, 0).load()).mod(Data.zeroExtend(WORD, load_op.access().load())));
    if ((load_address_error.access().load().compareTo(Data.valueOf(WORD, 0x0)) > 0)) {
      C0_EPC.access().store(CIA.access().load());
      Execution.exception("AddressError: mips32_load");
      Execution.trace("type = 0x%x", load_op.access().load().bigIntegerValue(false));
    }
    temp.access().store(Data.valueOf(WORD, 0x8).multiply(Data.zeroExtend(WORD, temp_byte_offset.access().bitField(1, 0).load())));
    temp0.access().store(load_temp32.access().load().shiftRight(temp.access().load()));
    if (signed_op.access().load().equals(Data.valueOf(BYTE, 0x1))) {
      if (load_op.access().load().equals(Data.valueOf(Type.CARD(3), 0x4))) {
        rt.access().store(load_temp32.access().load());
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(3), 0x2))) {
        rt.access().store(Data.signExtend(WORD, temp0.access().bitField(15, 0).load()));
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(3), 0x1))) {
        rt.access().store(Data.signExtend(WORD, temp0.access().bitField(7, 0).load()));
      } else {
        Execution.exception("mips32_load signed: error");
      }
    } else {
      if (load_op.access().load().equals(Data.valueOf(Type.CARD(3), 0x4))) {
        rt.access().store(load_temp32.access().load());
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(3), 0x2))) {
        rt.access().store(Data.zeroExtend(WORD, temp0.access().bitField(15, 0).load()));
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(3), 0x1))) {
        rt.access().store(Data.zeroExtend(WORD, temp0.access().bitField(7, 0).load()));
      } else {
        Execution.exception("mips32_load unsigned: error");
      }
    }
  }
}